"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var http = require("http");
var net = require("net");
var http_message_1 = require("./http-message");
var AuthenticationState;
(function (AuthenticationState) {
    AuthenticationState[AuthenticationState["unknown"] = 0] = "unknown";
    AuthenticationState[AuthenticationState["noAuthentication"] = 1] = "noAuthentication";
    AuthenticationState[AuthenticationState["firstRound"] = 2] = "firstRound";
    AuthenticationState[AuthenticationState["secondRound"] = 3] = "secondRound";
})(AuthenticationState || (AuthenticationState = {}));
var State = (function () {
    function State() {
        this.internalState = AuthenticationState.unknown;
    }
    State.prototype.nextActionOnConnection = function (req) {
        var headers = req.headers;
        console.info(headers);
        function authenticationRequiredResponse() {
            var res = new http_message_1.HttpMessage();
            res.setHttpVersion(req.httpVersion);
            res.setStatusCode(407);
            res.setStatusText('ProxyAuthentication required');
            res.addHeader('Proxy-Authenticate', 'Negotiate');
            res.addHeader('Proxy-Authenticate', 'NTLM');
            res.addHeader('Connection', 'close');
            res.addHeader('Proxy-Connection', 'close');
            return res;
        }
        function firstRoundResponse() {
            var res = new http_message_1.HttpMessage();
            res.setHttpVersion(req.httpVersion);
            res.setStatusCode(407);
            res.setStatusText('ProxyAuthentication required');
            res.addHeader('Proxy-Authenticate', 'NTLM #hash');
            res.addHeader('Connection', 'keep-alive');
            res.addHeader('Proxy-Connection', 'keep-alive');
            return res;
        }
        function secondRoundResponse() {
            var res = new http_message_1.HttpMessage();
            res.setHttpVersion(req.httpVersion);
            res.setStatusCode(200);
            res.setStatusText('Connection established');
            return res;
        }
        if (headers['proxy-authorization'] === undefined) {
            this.internalState = AuthenticationState.noAuthentication;
            console.info('no proxy-authenticate');
            var resp = authenticationRequiredResponse();
            console.info('resp %j', resp);
            return resp.createMessage();
        }
        else {
            switch (this.internalState) {
                case AuthenticationState.unknown:
                    this.internalState = AuthenticationState.firstRound;
                    return firstRoundResponse().createMessage();
                case AuthenticationState.firstRound:
                    this.internalState = AuthenticationState.secondRound;
                    return secondRoundResponse().createMessage();
                case AuthenticationState.secondRound:
                    break;
            }
        }
    };
    State.prototype.shouldClose = function () {
        return this.internalState === AuthenticationState.noAuthentication;
    };
    return State;
}());
var Socket = (function (_super) {
    __extends(Socket, _super);
    function Socket() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return Socket;
}(net.Socket));
var ProxyServer = (function () {
    function ProxyServer() {
    }
    ProxyServer.prototype.start = function (port) {
        var _this = this;
        this.server = http.createServer();
        this.server.on('connection', function (socket) {
            var s = socket;
            if (s.applicationState === undefined) {
                console.info();
                console.info('new socket');
                s.applicationState = new State();
            }
            else {
                console.info('rewirting socket');
            }
        });
        this.server.on('connect', function (req, socket, head) {
            console.info('CONNECT event');
            console.info('headers %j', req.headers);
            console.info('state %j', socket.applicationState);
            socket.write(socket.applicationState.nextActionOnConnection(req));
            console.info('New socket state: %j', socket.applicationState);
            if (socket.applicationState.shouldClose()) {
                console.info('tear down socket');
                socket.destroy();
            }
            else {
                _this.server.emit('connection', socket);
            }
        });
        this.server.on('request', function (req, res) {
            console.info('REQUEST %j', req);
            var socket = req.socket;
            res.write(socket.applicationState.nextActionOnConnection(req));
        });
        this.server.on('clientError', function (exception, socket) {
            console.error('ProxyServer: client error: %j', exception.message);
            socket.destroy(exception);
        });
        this.server.on('upgrade', function (request, socket, head) {
            console.info('Wants to upgrade -> teardown');
            socket.destroy("No upgrades today");
        });
        this.server.listen(port);
        console.info("Server started on " + port);
    };
    ProxyServer.prototype.stop = function () {
        if (this.server) {
            this.server.close();
        }
    };
    return ProxyServer;
}());
exports.ProxyServer = ProxyServer;
//# sourceMappingURL=proxy-server.js.map